-- | Pictures and Colours inspired by "CodeWorld"
module ThreepennyPictures(
  -- * Pictures
  Picture,
  (&),pictures,blank,path,
  -- | Pixel coordinates. (0,0) is the center of the canvas.
  UI.Point,
  coloured,colored,
  --text,
  -- ** Colours
  Colour,Color,
  black,white,red,green,blue,cyan,magenta,yellow,orange,gray,grey,
  -- ** Drawing pictures
  draw,drawOnTop,
  ) where
import Control.Monad(unless)
import qualified Graphics.UI.Threepenny as UI
import Graphics.UI.Threepenny.Core as UI
import Numeric(showHex)

newtype Picture = P (Bool->UI.Point->UI.Canvas->UI ())

instance Monoid Picture where
  mempty = blank
  mappend = (&)
  
infixr 0 &
(&) :: Picture -> Picture -> Picture
P p1 & P p2 = P (\b o c->p1 b o c >> p2 b o c)

pictures :: [Picture] -> Picture
pictures = mconcat

draw,drawOnTop :: Picture -> (Double, Double) -> UI.Canvas -> UI ()
draw p size c = do UI.clearCanvas c
                   drawOnTop p size c

drawOnTop (P f) (w,h) c = f False origin c
  where origin = (w/2,h/2)


colored, coloured :: Colour -> Picture -> Picture
colored = coloured
coloured (C color) (P p) = P $ \ b o c ->
                           do unless b (c # set' UI.strokeStyle color)
                              c # p True o

path :: [UI.Point] -> Picture
path [] = blank
path (p:ps) = P path
  where
    path b o c = do c # UI.beginPath
                    c # UI.moveTo (conv o p)
                    mapM_ (flip UI.lineTo c) (map (conv o) ps)
                    c # UI.stroke

-- | Convert form coordinates with the origin in the center of the canvas
-- to coordinates with origin in the top left corder
conv :: UI.Point -> UI.Point -> UI.Point
conv (x0,y0) (x,y) = (x0+x,y0-y)

--text :: String -> UI.Point -> Picture
--text txt p = P (\b o c->UI.fillText txt (conv o p) c)

blank :: Picture
blank = P (\_ _ _->return ())

--------------------------------------------------------------------------------

newtype Colour = C CSS_Colour deriving (Show)
type Color = Colour

type CSS_Colour = String


black = C "black"
white = C "white"
red = C "red"
green = C "green"
blue = C "blue"
cyan = C "cyan"
magenta = C "magenta"
yellow = C "yellow"
orange = C "orange"

gray = grey

grey :: Double -> Colour
grey d = C ("#"++h++h++h)
  where
    s = showHex (floor (d*255) `min` 255) ""
    h = if length s==1 then "0"++s else s
